import ObjetoDinamico from './ObjetoDinamico.js';

/**
 * Classe Ponto
 * 
 * Utilizada para representar um Ponto no desenho geométrico
 * 
 * @author Robertino Mendes Santiago Jr <robertino@ufpr.br>
 */
export default class Ponto extends ObjetoDinamico {

    /**
     * Método construtor da classe Ponto
     * @param {number} x Posição relativa ao eixo x onde o ponto será criado
     * @param {number} y Posição relativa ao eixo y onde o ponto será criado
     */
    constructor(x, y) {
        super();
        this._x = x;
        this._y = y;
        this._raio = 4;
        this._espessura = 1;
        this._selecionado = false;

        this._elemento = this.criaPonto();
    }

    /**
     * Método que devolve o elemento Ponto.
     * @return {Object} Retorna o elemento que representa um Ponto em formato SVG
     */
    draw() {
        return this._elemento;
    }

    /**
     * Verifica se existe um ponto na posição especificada por x e y
     * @param {number} x - Valor da posição no eixo x
     * @param {number} y - Valor da posição no eixo y
     * @return {boolean} Retorna verdadeiro se existe um ponto na posição especificada. Caso contrário, retorna falso.
     */
    contains(x, y) {
        let xDiff = Math.abs(this.x - x);
        let yDiff = Math.abs(this.y - y);
        let dist = Math.sqrt(Math.pow(xDiff,2) + Math.pow(yDiff,2)); 
        return dist <= this.raio;
    }

    /**
     * Método que cria um elemento SVG que representa um Ponto
     * @return {Object} Retorna o elemento SVG criado que representa o Ponto
     */
    criaPonto() {
        let elemento = document.createElementNS(this.svgNS,'circle');
        elemento.setAttribute('id','mycircle');
        elemento.setAttribute('cx',this.x);
        elemento.setAttribute('cy',this.y);
        elemento.setAttribute('r',this.raio);
        elemento.setAttribute('fill','lightgreen');
        elemento.setAttribute('stroke','black');
        elemento.setAttribute('stroke-width', this.espessura);
        return elemento;
    }

    /**
     * Método que realiza a movimentação do Ponto
     * @param {number} x Posição relativa ao eixo x onde o ponto será movimentado
     * @param {number} y Posição relativa ao eixo y onde o ponto será movimentado
     */
    mover(x, y) {
        this.x = x;
        this.y = y;
    }

    /**
     * Devolve o valor da propriedade x
     * @return {number} Valor da propriedade x
     */
    get x() {
        return this._x;
    }

    /**
     * Atribui um valor recebido por parâmetro à propriedade x
     * @param {number} x Posição relativa ao eixo x
     */
    set x(x) {
        this._x = x;
        this._elemento.setAttribute('cx', x);
    }

    /**
     * Devolve o valor da propriedade y
     * @return {number} Valor da propriedade y
     */
    get y() {
        return this._y;
    }

    /**
     * Atribui um valor recebido por parâmetro à propriedade y
     * @param {number} y Posição relativa ao eixo y
     */
    set y(y) {
        this._y = y;
        this._elemento.setAttribute('cy', y);
    }

    /**
     * Devolve o valor da propriedade raio
     * @return {number} Valor da propriedade raio
     */
    get raio() {
        return this._raio;
    }

    /**
     * Atribui um valor recebido por parâmetro à propriedade raio
     * @param {number} raio Valor do raio do Ponto
     */
    set raio(raio) {
        this._raio = raio;
        this._elemento.setAttribute('r', raio);
    }

    /**
     * Devolve o valor da propriedade espessura da borda
     * @return {number} Valor da propriedade da espessura da borda
     */
    get espessura() {
        return this._espessura;
    }

    /**
     * Atribui um valor recebido por parâmetro à propriedade espessura
     * @param {number} espessura Valor da espessura da borda
     */
    set espessura(espessura) {
        this._espessura = espessura; 
        this._elemento.setAttribute('stroke-width', espessura);
    }

    /**
     * Devolve o valor da propriedade selecionado
     * @return {boolean} Valor da propriedade selecionado
     */
    get selecionado() {
        return this._selecionado;
    }

    /**
     * Atribui um valor recebido por parâmetro à propriedade selecionado
     * @param {boolean} selecionado Verdadeiro se o objeto está selecionado ou falso caso contrário
     */
    set selecionado(selecionado) {
        this._selecionado = selecionado;
        if (selecionado) {
            this._elemento.setAttribute('stroke', 'red');
        } else {
            this._elemento.setAttribute('stroke', 'black');
        }
    }
    
}